package net.es.oscars.policy.client;

import java.io.Serializable;
import java.util.List;

import org.w3c.dom.Element;

/**
 * Stores Requester data in W3C DOM elements. Elements are serializable so this
 * element is used in requests to the RMI interface. The W3C DOM elements conform 
 * to SAML2 elements but OpenSAML types are not serializable so this class may be 
 * useful in such situations.
 * 
 * @author Andrew Lake (alake@internet2.edu)
 */
public class DOMRequester implements Serializable{
    private Element subject;
    private Element subjectAuthentication;
    private List<Element> assertions;
    /**
     * Returns the requester subject. Conforms to SAML2 Subject definition
     * 
     * @return the subject as a W3C DOM element
     */
    public Element getSubject() {
        return this.subject;
    }
    
    /**
     * Sets the requester subject. Conforms to SAML2 Subject definition
     * 
     * @param the subject as a W3C DOM element
     */
    public void setSubject(Element subject) {
        this.subject = subject;
    }
    
    /**
     * Returns the value of the SubjectAuthentication as a W3C DOM element. This 
     * element conforms to the definition of NameID. It does NOT include
     * the <subjectAuthentication> tag.
     * 
     * @return the value of the SubjectAuthentication as a W3C DOM element
     */
    public Element getSubjectAuthentication() {
        return this.subjectAuthentication;
    }
    
    /**
     * Sets the value of the SubjectAuthentication as a W3C DOM element. This 
     * element conforms to the definition of NameID. It does NOT include
     * the <subjectAuthentication> tag.
     * 
     * @param subjectAuthentication the subjectAuthentication to set
     */
    public void setSubjectAuthentication(Element subjectAuthentication) {
        this.subjectAuthentication = subjectAuthentication;
    }
    
    /**
     * Returns a list of Assertions as W3C DOM Elements. Each element must 
     * conform to the SAML2 Assertion definition.
     * 
     * @return the list of Assertions as W3C DOM types
     */
    public List<Element> getAssertions() {
        return this.assertions;
    }
    
    /**
     * Sets  a list of Assertions as W3C DOM Elements. Each element must 
     * conform to the SAML2 Assertion definition.
     * 
     * @param assertions the list of Assertions as W3C DOM types
     */
    public void setAssertions(List<Element> assertions) {
        this.assertions = assertions;
    }
}
