package net.es.oscars.policy.examples;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axis2.databinding.ADBException;
import org.ogf.schema.network.topology.ctrlplane.CtrlPlaneHopContent;
import org.ogf.schema.network.topology.ctrlplane.CtrlPlaneLinkContent;
import org.ogf.schema.network.topology.ctrlplane.CtrlPlanePathContent;
import org.ogf.schema.network.topology.ctrlplane.CtrlPlaneSwcapContent;
import org.ogf.schema.network.topology.ctrlplane.CtrlPlaneSwitchingCapabilitySpecificInfo;
import org.ogf.schema.network.topology.ctrlplane.Link;
import org.ogf.schema.network.topology.ctrlplane.Path;

import net.es.oscars.policy.axis2.Axis2PolicyClient;
import net.es.oscars.policy.client.CheckPolicyRequest;
import net.es.oscars.policy.client.CheckPolicyResponse;
import net.es.oscars.policy.client.PolicyClientException;
import net.es.oscars.policy.client.Requester;
import net.es.oscars.wsdlTypes.CreateReservation;
import net.es.oscars.wsdlTypes.Layer3Info;
import net.es.oscars.wsdlTypes.LocalDetails;
import net.es.oscars.wsdlTypes.MplsInfo;
import net.es.oscars.wsdlTypes.PathInfo;
import net.es.oscars.wsdlTypes.ResCreateContent;
import net.es.oscars.wsdlTypes.ReservationResource;
import net.es.oscars.wsdlTypes.ReservationResourceType;

public class ExamplePolicyClient {
    public static CtrlPlaneLinkContent createLink(String urn, String vlan){
        CtrlPlaneLinkContent link = new CtrlPlaneLinkContent();
        link.setId(urn);
        link.setTrafficEngineeringMetric("10");
        CtrlPlaneSwcapContent swcap = new CtrlPlaneSwcapContent();
        CtrlPlaneSwitchingCapabilitySpecificInfo swcapInfo = new CtrlPlaneSwitchingCapabilitySpecificInfo();
        swcap.setEncodingType("ethernet");
        swcap.setSwitchingcapType("l2sc");
        swcapInfo.setInterfaceMTU(9000);
        swcapInfo.setVlanRangeAvailability(vlan);
        swcap.setSwitchingCapabilitySpecificInfo(swcapInfo);
        link.setSwitchingCapabilityDescriptors(swcap);
        
        return link;
    }
    public static void main(String[] args){
        try {
            String catalinaHome = System.getenv("CATALINA_HOME");
            Axis2PolicyClient policyClient = new Axis2PolicyClient();
            CheckPolicyRequest<Requester,OMElement> request = policyClient.createRequest();
            String url = "http://127.0.0.1:8080/axis2/services/OSCARSPolicy";
            if(args.length > 0){
                url = args[0];
            }
            
            //configure client
            policyClient.getConfiguration().put(Axis2PolicyClient.REPO_CONFIG, catalinaHome + "/shared/classes/repo/");
            policyClient.getConfiguration().put(Axis2PolicyClient.AXIS2_CONFIG, catalinaHome + "/shared/classes/repo/axis2-norampart.xml");
            
            //add requesters and action
            Requester originator = new Requester();
            originator.setSubject("C=US, ST=Michigan, O=Internet2, OU=R&D, CN=Andrew Lake, L=Ann Arbor", null);
           // originator.setSubjectAuthentication("http://test-idc.internet2.edu:8443/axis2/service/OSCARS");
            request.getRequesters().add(originator);
            request.setAction("urn:dcn:oscars:action:createReservation");
            
            //create resource
            ReservationResourceType resResource = new ReservationResourceType();
            LocalDetails localDetails = new LocalDetails();
            resResource.setGlobalReservationId("dcn.internet2.edu-2");
            PathInfo pathInfo = new PathInfo();
            CtrlPlanePathContent path = new CtrlPlanePathContent();
            path.setId("path1");
            CtrlPlaneHopContent firstHop = new CtrlPlaneHopContent();
            firstHop.setId("1");
            
            firstHop.setLink(createLink("urn:ogf:network:domain=anna.internet2.edu:node=raptor2:port=1-1-1:link=*", "500"));
           //firstHop.setLinkIdRef("urn:ogf:network:domain=anna.internet2.edu:node=raptor1:port=1-1-1:link=*");
            path.addHop(firstHop);
            CtrlPlaneHopContent lastHop = new CtrlPlaneHopContent();
            lastHop.setId("2");
            lastHop.setLink(createLink("urn:ogf:network:domain=anna.internet2.edu:node=raptor2:port=1-1-2:link=*", "3001"));
            //lastHop.setLinkIdRef("urn:ogf:network:domain=anna.internet2.edu:node=raptor2:port=1-1-2:link=*");
            path.addHop(lastHop);
            pathInfo.setPath(path);
            resResource.setDescription("[PRODUCTION] test");
            resResource.setStartTime((System.currentTimeMillis())/1000);
            resResource.setEndTime((System.currentTimeMillis()+3650000+100000)/1000);
            MplsInfo mplsInfo = new MplsInfo();
            mplsInfo.setBurstLimit(999);
            mplsInfo.setLspClass("3");
            pathInfo.setMplsInfo(mplsInfo);
            Layer3Info l3Info = new Layer3Info();
            l3Info.setSrcHost("host1.geant.edu");
            l3Info.setDestHost("host2.internet2.edu");
            l3Info.setSrcIpPort(1025);
            l3Info.setDestIpPort(1023);
            l3Info.setProtocol("tcp");
            l3Info.setDscp("test");
            pathInfo.setLayer3Info(l3Info);
            pathInfo.setPathSetupMode("timer-automatic");
            resResource.setPathInfo(pathInfo);
            resResource.setBandwidth(400);
            OMElement omEvent = resResource.getOMElement(ReservationResource.MY_QNAME, OMAbstractFactory.getOMFactory());
            request.getResources().add(omEvent);
            
            localDetails.addExtraElement(path.getOMElement(Path.MY_QNAME, OMAbstractFactory.getOMFactory()));
            resResource.setLocalDetails(localDetails);
            //send and print
            CheckPolicyResponse response = policyClient.checkPolicy(url, request);
            System.out.println("Allow: " + response.getAllow());
            System.out.println("Reason: " + response.getReason());
        } catch (PolicyClientException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (ADBException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
}
